@@ -33,6 +33,7 @@
 # Copyright (c) 2019 Andrzej Klajnert <github@aklajnert.pl>
 # Copyright (c) 2019 Pascal Corpet <pcorpet@users.noreply.github.com>
 # Copyright (c) 2020 GergelyKalmar <gergely.kalmar@logikal.jp>
+# Copyright (c) 2021 Yu Shao, Pang <p.yushao2@gmail.com>
 # Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 # Copyright (c) 2021 yushao2 <36848472+yushao2@users.noreply.github.com>
 # Copyright (c) 2021 Konstantina Saketou <56515303+ksaketou@users.noreply.github.com>
@@ -960,10 +961,34 @@ a metaclass class method.",
         for assign_attr in node.nodes_of_class(astroid.AssignAttr):
             if not is_attr_private(assign_attr.attrname):
                 continue
+
+            # Logic for checking false positive when using __new__,
+            # Get the returned object names of the __new__ magic function
+            # Then check if the attribute was consumed in other instance methods
+            acceptable_obj_names: List[str] = ["self"]
+            scope = assign_attr.scope()
+            if isinstance(scope, astroid.FunctionDef) and scope.name == "__new__":
+                acceptable_obj_names.extend(
+                    [
+                        return_node.value.name
+                        for return_node in scope.nodes_of_class(astroid.Return)
+                        if isinstance(return_node.value, astroid.Name)
+                    ]
+                )
+
             for attribute in node.nodes_of_class(astroid.Attribute):
-                if (
-                    attribute.attrname == assign_attr.attrname
-                    and attribute.expr.name == assign_attr.expr.name
+                if attribute.attrname == assign_attr.attrname and (
+                    (
+                        # If assigned to cls.attrib, can be accessed by cls/self
+                        assign_attr.expr.name == "cls"
+                        and attribute.expr.name in ["cls", "self"]
+                    )
+                    # If assigned to self.attrib, can only be accessed by self
+                    # Or if __new__ was used, the returned object names are acceptable
+                    or (
+                        assign_attr.expr.name in acceptable_obj_names
+                        and attribute.expr.name == "self"
+                    )
                 ):
                     break
             else:
