@@ -1,8 +1,7 @@
-# -*- coding: utf-8 -*-
 # Copyright (c) 2006-2011, 2013-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
 # Copyright (c) 2011-2014 Google, Inc.
 # Copyright (c) 2012 Tim Hatch <tim@timhatch.com>
-# Copyright (c) 2013-2018 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2013-2020 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2014 Brett Cannon <brett@python.org>
 # Copyright (c) 2014 Arun Persaud <arun@nubati.net>
 # Copyright (c) 2015 Rene Zhang <rz99@cornell.edu>
@@ -13,25 +12,34 @@
 # Copyright (c) 2016 Jakub Wilk <jwilk@jwilk.net>
 # Copyright (c) 2017 Łukasz Rogalski <rogalski.91@gmail.com>
 # Copyright (c) 2017 Martin von Gagern <gagern@google.com>
-# Copyright (c) 2018 Mike Frysinger <vapier@gmail.com>
+# Copyright (c) 2018 Lucas Cimon <lucas.cimon@gmail.com>
 # Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
+# Copyright (c) 2018 Natalie Serebryakova <natalie.serebryakova@Natalies-MacBook-Pro.local>
+# Copyright (c) 2018 Sushobhit <31987769+sushobhit27@users.noreply.github.com>
+# Copyright (c) 2018 Carey Metcalfe <carey@cmetcalfe.ca>
+# Copyright (c) 2018 Mike Frysinger <vapier@gmail.com>
 # Copyright (c) 2018 Alexander Todorov <atodorov@otb.bg>
-# Copyright (c) 2018 Ville Skyttä <ville.skytta@upcloud.com>
+# Copyright (c) 2018 Ville Skyttä <ville.skytta@iki.fi>
+# Copyright (c) 2019, 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2019 Djailla <bastien.vallet@gmail.com>
+# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
+# Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>
+# Copyright (c) 2020 Ram Rachum <ram@rachum.com>
+# Copyright (c) 2020 Anthony Sottile <asottile@umich.edu>
+# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
-# For details: https://github.com/PyCQA/pylint/blob/master/COPYING
+# For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
 
 """Checks for various exception related errors."""
 import builtins
 import inspect
-import sys
 import typing
 
 import astroid
 
-from pylint import checkers
+from pylint import checkers, interfaces
 from pylint.checkers import utils
-from pylint import interfaces
 
 
 def _builtin_exceptions():
@@ -47,7 +55,7 @@ def _annotated_unpack_infer(stmt, context=None):
     Recursively generate nodes inferred by the given statement.
     If the inferred value is a list or a tuple, recurse on the elements.
     Returns an iterator which yields tuples in the format
-    ('original node', 'infered node').
+    ('original node', 'inferred node').
     """
     if isinstance(stmt, (astroid.List, astroid.Tuple)):
         for elt in stmt.elts:
@@ -55,10 +63,10 @@ def _annotated_unpack_infer(stmt, context=None):
             if inferred and inferred is not astroid.Uninferable:
                 yield elt, inferred
         return
-    for infered in stmt.infer(context):
-        if infered is astroid.Uninferable:
+    for inferred in stmt.infer(context):
+        if inferred is astroid.Uninferable:
             continue
-        yield stmt, infered
+        yield stmt, inferred
 
 
 def _is_raising(body: typing.List) -> bool:
@@ -69,11 +77,9 @@ def _is_raising(body: typing.List) -> bool:
     return False
 
 
-PY3K = sys.version_info >= (3, 0)
-OVERGENERAL_EXCEPTIONS = ("Exception",)
-BUILTINS_NAME = builtins.__name__
+OVERGENERAL_EXCEPTIONS = ("BaseException", "Exception")
 
-MSGS = {
+MSGS = {  # pylint: disable=consider-using-namedtuple-or-dataclass
     "E0701": (
         "Bad except clauses order (%s)",
         "bad-except-order",
@@ -146,6 +152,15 @@ MSGS = {
         "immediately. Remove the raise operator or the entire "
         "try-except-raise block!",
     ),
+    "W0707": (
+        "Consider explicitly re-raising using the 'from' keyword",
+        "raise-missing-from",
+        "Python 3's exception chaining means it shows the traceback of the "
+        "current exception, but also the original exception. Not using `raise "
+        "from` makes the traceback inaccurate, because the message implies "
+        "there is a bug in the exception-handling code itself, which is a "
+        "separate situation than wrapping an exception.",
+    ),
     "W0711": (
         'Exception to catch is the result of a binary "%s" operation',
         "binary-op-exception",
@@ -160,6 +175,13 @@ MSGS = {
         "constructor, the first of them a string literal containing what "
         "appears to be placeholders intended for formatting",
     ),
+    "W0716": (
+        "Invalid exception operation. %s",
+        "wrong-exception-operation",
+        "Used when an operation is done against an exception, but the operation "
+        "is not valid for the exception in question. Usually emitted when having "
+        "binary operations between exceptions in except handlers.",
+    ),
 }
 
 
@@ -224,33 +246,9 @@ class ExceptionRaiseLeafVisitor(BaseVisitor):
         if not utils.inherit_from_std_ex(cls) and utils.has_known_bases(cls):
             if cls.newstyle:
                 self._checker.add_message("raising-non-exception", node=self._node)
-            else:
-                self._checker.add_message("nonstandard-exception", node=self._node)
 
-    def visit_tuple(self, tuple_node):
-        if PY3K or not tuple_node.elts:
-            self._checker.add_message("raising-bad-type", node=self._node, args="tuple")
-            return
-
-        # On Python 2, using the following is not an error:
-        #    raise (ZeroDivisionError, None)
-        #    raise (ZeroDivisionError, )
-        # What's left to do is to check that the first
-        # argument is indeed an exception. Verifying the other arguments
-        # is not the scope of this check.
-        first = tuple_node.elts[0]
-        inferred = utils.safe_infer(first)
-        if not inferred or inferred is astroid.Uninferable:
-            return
-
-        if (
-            isinstance(inferred, astroid.Instance)
-            and inferred.__class__.__name__ != "Instance"
-        ):
-            # TODO: explain why
-            self.visit_default(tuple_node)
-        else:
-            self.visit(inferred)
+    def visit_tuple(self, _):
+        self._checker.add_message("raising-bad-type", node=self._node, args="tuple")
 
     def visit_default(self, node):
         name = getattr(node, "name", node.__class__.__name__)
@@ -281,23 +279,25 @@ class ExceptionsChecker(checkers.BaseChecker):
 
     def open(self):
         self._builtin_exceptions = _builtin_exceptions()
-        super(ExceptionsChecker, self).open()
+        super().open()
 
     @utils.check_messages(
-        "nonstandard-exception",
         "misplaced-bare-raise",
         "raising-bad-type",
         "raising-non-exception",
         "notimplemented-raised",
         "bad-exception-context",
         "raising-format-tuple",
+        "raise-missing-from",
     )
     def visit_raise(self, node):
         if node.exc is None:
             self._check_misplaced_bare_raise(node)
             return
 
-        if PY3K and node.cause:
+        if node.cause is None:
+            self._check_raise_missing_from(node)
+        else:
             self._check_bad_exception_context(node)
 
         expr = node.exc
@@ -332,7 +332,7 @@ class ExceptionsChecker(checkers.BaseChecker):
         if not current or not isinstance(current.parent, expected):
             self.add_message("misplaced-bare-raise", node=node)
 
-    def _check_bad_exception_context(self, node):
+    def _check_bad_exception_context(self, node: astroid.Raise) -> None:
         """Verify that the exception context is properly set.
 
         An exception context can be only `None` or an exception.
@@ -349,6 +349,37 @@ class ExceptionsChecker(checkers.BaseChecker):
         ):
             self.add_message("bad-exception-context", node=node)
 
+    def _check_raise_missing_from(self, node: astroid.Raise) -> None:
+        if node.exc is None:
+            # This is a plain `raise`, raising the previously-caught exception. No need for a
+            # cause.
+            return
+        # We'd like to check whether we're inside an `except` clause:
+        containing_except_node = utils.find_except_wrapper_node_in_scope(node)
+        if not containing_except_node:
+            return
+        # We found a surrounding `except`! We're almost done proving there's a
+        # `raise-missing-from` here. The only thing we need to protect against is that maybe
+        # the `raise` is raising the exception that was caught, possibly with some shenanigans
+        # like `exc.with_traceback(whatever)`. We won't analyze these, we'll just assume
+        # there's a violation on two simple cases: `raise SomeException(whatever)` and `raise
+        # SomeException`.
+        if containing_except_node.name is None:
+            # The `except` doesn't have an `as exception:` part, meaning there's no way that
+            # the `raise` is raising the same exception.
+            self.add_message("raise-missing-from", node=node)
+        elif isinstance(node.exc, astroid.Call) and isinstance(
+            node.exc.func, astroid.Name
+        ):
+            # We have a `raise SomeException(whatever)`.
+            self.add_message("raise-missing-from", node=node)
+        elif (
+            isinstance(node.exc, astroid.Name)
+            and node.exc.name != containing_except_node.name.name
+        ):
+            # We have a `raise SomeException`.
+            self.add_message("raise-missing-from", node=node)
+
     def _check_catching_non_exception(self, handler, exc, part):
         if isinstance(exc, astroid.Tuple):
             # Check if it is a tuple of exceptions.
@@ -364,7 +395,7 @@ class ExceptionsChecker(checkers.BaseChecker):
                 return
 
         if not isinstance(exc, astroid.ClassDef):
-            # Don't emit the warning if the infered stmt
+            # Don't emit the warning if the inferred stmt
             # is None, but the exception handler is something else,
             # maybe it was redefined.
             if isinstance(exc, astroid.Const) and exc.value is None:
@@ -399,18 +430,25 @@ class ExceptionsChecker(checkers.BaseChecker):
                 )
 
     def _check_try_except_raise(self, node):
-        def gather_exceptions_from_handler(handler):
-            exceptions = []
+        def gather_exceptions_from_handler(
+            handler,
+        ) -> typing.Optional[typing.List[astroid.node_classes.NodeNG]]:
+            exceptions: typing.List[astroid.node_classes.NodeNG] = []
             if handler.type:
                 exceptions_in_handler = utils.safe_infer(handler.type)
                 if isinstance(exceptions_in_handler, astroid.Tuple):
-                    exceptions = {
-                        exception
-                        for exception in exceptions_in_handler.elts
-                        if isinstance(exception, astroid.Name)
-                    }
+                    exceptions = list(
+                        {
+                            exception
+                            for exception in exceptions_in_handler.elts
+                            if isinstance(exception, astroid.Name)
+                        }
+                    )
                 elif exceptions_in_handler:
                     exceptions = [exceptions_in_handler]
+                else:
+                    # Break when we cannot infer anything reliably.
+                    return None
             return exceptions
 
         bare_raise = False
@@ -424,16 +462,15 @@ class ExceptionsChecker(checkers.BaseChecker):
 
                 excs_in_current_handler = gather_exceptions_from_handler(handler)
                 if not excs_in_current_handler:
-                    bare_raise = False
                     break
-
+                if excs_in_bare_handler is None:
+                    # It can be `None` when the inference failed
+                    break
                 for exc_in_current_handler in excs_in_current_handler:
                     inferred_current = utils.safe_infer(exc_in_current_handler)
                     if any(
-                        utils.is_subclass_of(
-                            utils.safe_infer(exc_in_bare_handler), inferred_current
-                        )
-                        for exc_in_bare_handler in excs_in_bare_handler
+                        utils.is_subclass_of(utils.safe_infer(e), inferred_current)
+                        for e in excs_in_bare_handler
                     ):
                         bare_raise = False
                         break
@@ -445,8 +482,29 @@ class ExceptionsChecker(checkers.BaseChecker):
                     bare_raise = True
                     handler_having_bare_raise = handler
                     excs_in_bare_handler = gather_exceptions_from_handler(handler)
-        if bare_raise:
-            self.add_message("try-except-raise", node=handler_having_bare_raise)
+        else:
+            if bare_raise:
+                self.add_message("try-except-raise", node=handler_having_bare_raise)
+
+    @utils.check_messages("wrong-exception-operation")
+    def visit_binop(self, node):
+        if isinstance(node.parent, astroid.ExceptHandler):
+            # except (V | A)
+            suggestion = "Did you mean '({}, {})' instead?".format(
+                node.left.as_string(),
+                node.right.as_string(),
+            )
+            self.add_message("wrong-exception-operation", node=node, args=(suggestion,))
+
+    @utils.check_messages("wrong-exception-operation")
+    def visit_compare(self, node):
+        if isinstance(node.parent, astroid.ExceptHandler):
+            # except (V < A)
+            suggestion = "Did you mean '({}, {})' instead?".format(
+                node.left.as_string(),
+                ", ".join(operand.as_string() for _, operand in node.ops),
+            )
+            self.add_message("wrong-exception-operation", node=node, args=(suggestion,))
 
     @utils.check_messages(
         "bare-except",
@@ -505,7 +563,7 @@ class ExceptionsChecker(checkers.BaseChecker):
 
                     for previous_exc in exceptions_classes:
                         if previous_exc in exc_ancestors:
-                            msg = "%s is an ancestor class of %s" % (
+                            msg = "{} is an ancestor class of {}".format(
                                 previous_exc.name,
                                 exc.name,
                             )
