@@ -33,10 +33,11 @@
 # Copyright (c) 2020 Federico Bond <federicobond@gmail.com>
 # Copyright (c) 2020 Athos Ribeiro <athoscr@fedoraproject.org>
 # Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
+# Copyright (c) 2021 bot <bot@noreply.github.com>
 # Copyright (c) 2021 Tiago Honorato <tiagohonorato1@gmail.com>
 # Copyright (c) 2021 tiagohonorato <61059243+tiagohonorato@users.noreply.github.com>
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
-# For details: https://github.com/PyCQA/pylint/blob/master/LICENSE
+# For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
 
 """Check Python 2 code for Python 2/3 source-compatible issues."""
 import itertools
@@ -907,16 +908,14 @@ class Python3Checker(checkers.BaseChecker):
     )
 
     _python_2_tests = frozenset(
-        [
-            astroid.extract_node(x).repr_tree()
-            for x in [
-                "sys.version_info[0] == 2",
-                "sys.version_info[0] < 3",
-                "sys.version_info == (2, 7)",
-                "sys.version_info <= (2, 7)",
-                "sys.version_info < (3, 0)",
-            ]
-        ]
+        astroid.extract_node(x).repr_tree()
+        for x in (
+            "sys.version_info[0] == 2",
+            "sys.version_info[0] < 3",
+            "sys.version_info == (2, 7)",
+            "sys.version_info <= (2, 7)",
+            "sys.version_info < (3, 0)",
+        )
     )
 
     def __init__(self, *args, **kwargs):
